<?php

	require_once('ttmailer_config.php');
	require_once('ttmailer_lib.php');

############################### The Sub-Routine Order #########################
#
# Sub routines in order of their processing your form data. Comment out
# the optional ones you don't want to activate, by putting a # and a space in
# front of them, like this :
#     # cookie_bake();
# It's a good idea to do this if you feel up to it, as the script will
# then run faster, and be less prone to errors. Some are already commented out below.
#
###############################################################################
#   WARNING!: Sub-routines calls marked 'ESSENTIAL' should be left alone!     #
###############################################################################

hack_filter();       # Optional. Filter all html-tags of all fields.
                     # If present, has to be BEFORE parse_data();

parse_data();        # ESSENTIAL. Parse the raw form data.

referer_check();     # Optional.  Check only your website is calling this script.
                     # Comment this option out if you are using the affiliate-
                     # tracking or unsubscribe-from-an-email functions.

required_check();    # Optional. Check required fields have been filled in.

email_check();       # Optional. Check basic validity of email address syntax.

# cookie_bake();       # Optional. For getting or setting a cookie.

# save_delimited();    # Optional. Save to database file

# subscription();      # Optional. Subscribe or unsubscribe to mailing list.

their_email();       # Optional. Send an email to the customer.

# sms_message();       # Optional. Send a text message to the webmaster's mobile 'phone. 

your_email();        # Optional. Send an email to the webmaster.

end_url();           # ESSENTIAL. Final script routines.

exit();              # ESSENTIAL. Script action ends.

##############################################################################
# COPYRIGHT NOTICE                                                           #
# TigerTom's Form Mailer. Version: 1.0, July 2006.                           #
#                                                                            #
# Copyright 2006 T. O' Donnell.  All Rights Reserved.                        #
# http://www.tigertom.com/contact.htm                                        #
#                                                                            #
# This Perl script may be used and modified free of charge by anyone so      #
# long as this copyright notice, the comments below, and the advertisement   #
# it generates remain intact. By using this code you agree to indemnify T. O'#
# Donnell, Inc. and T. O' Donnell from any liability that might arise from   #
# its use.                                                                   #
#                                                                            #
# Selling the code for this program without prior written consent is         #
# expressly forbidden.                                                       #
# You must get permission from T. O' Donnell and the other contributors      #
# before redistributing this software over the Internet or via any           #
# other medium.                                                              #
# You may not redistribute an altered version of this program in any form,   #
# via any medium.                                                            #
#                                                                            #
# In all cases copyright and header must remain intact.                      #
#                                                                            #
# This Copyright is in full effect in any country that has International     #
# Trade Agreements with the United Kingdom Of Great Britain.                 #
#                                                                            #
##############################################################################

########## Only would-be programmers should edit past this point! ##########

########################### The main sub-routines ###############################

# Get the form input :

$formData = array();

function hack_filter()
{
	$GLOBALS['hack_filter'] = 1;
}

function parse_data()
{
	$data = array();
	global $legit_length;

	if(strtolower($_SERVER['REQUEST_METHOD']) == "post")
	{
		$data = $_POST;
	} else
	{
		$data = $_GET;
		if(isset($_SERVER['QUERY_STRING']) && strlen($_SERVER['QUERY_STRING']) > $legit_length)
		{
			hack_alert('querystring', $_SERVER['QUERY_STRING']);
		}
	}

	$newData = array();

	foreach($data as $key => $value)
	{

		# filter all tags
		if(isset($GLOBALS['hack_filter']))
		{
			$value = strip_tags($value);
		}

		# prevent email-injection in all fields which go directly into header
		switch ($key) {
			case 'from_name':
			case 'from_email':
			case 'Recipient':
			case 'Subject':
			case 'Email':
			case 'Name':
				if (check_email_injection($value)) {
					hack_alert('emailinjection', $key .": [".$value."]");
				}
				break;
		}

		# convert some (after optional filter remaining) special-chars to other chars
		# Note, mail is sent as plain/text later so we do not use &lt; &gt; &quot; but [ ] *
		$value = str_replace (">", " ", $value);
		$value = str_replace ("<", " ", $value);
		$value = str_replace ("|", "/", $value);
		$value = str_replace ("}", "]", $value);
		$value = str_replace ("{", "[", $value);
		$value = str_replace ("^", " ", $value);
		$value = str_replace ("*", "+", $value);
		$value = str_replace ("~", " ", $value);
		$value = str_replace ("&", "+", $value);
		$value = str_replace ("%", "percent", $value);

		# EduJini: Wrapping can be done with the inbuilt wordwrap function
		global $wrap_at, $string_limit;
		if($key == 'message')
		{
			if(strlen($data['message']) > $string_limit)
			{
				$value = wordwrap($value, $wrap_at, "\r\n");
			}
		}

		$newData[$key] = $value;

	} # End foreach

	$data = $newData;
	$GLOBALS['advert'] = "This email was sent via software available from http://www.tigertom.com "
	                    ." designed by www.EduJini.in";

	global $formData;
	$formData = $data;

} # End read parse


###########################

# Send an email :

function your_email()
{
	global $subject, $formData;
	if(isset($formData['Name']) && isset($formData['Email']))
	{
		if(isset($formData['Subject']))
		{
			$subject = $formData['Subject'];
		}
		$from = $formData['Name'].' <'.$formData['Email'].'>';

		$headers = "";
		$headers .= 'From: '.$from."\r\n";

		# this is currently not used, check for email injection if you use it...
		##if(isset($GLOBALS['carbon']))
		##{
		##	$headers .= 'Cc: <'.$GLOBALS['carbon'].">\r\n";
		##}

		$message = "Dear ".$formData['from_name']."\r\n";
		$message .= "\r\n";
		$message .= "Here is the data submitted via your web form ".$_SERVER['HTTP_REFERER']." at (server time) ".date("r").":\r\n";
		$message .= "\r\n";

		$action = $formData['action'];
		if($action == 'subscribe')
		{
			if(isset($GLOBALS['duplic']) && $GLOBALS['duplic'])
			{
				$message .= $formData['Name'].' ('.$formData['Email'].') has tried to make a duplicate'."\r\n";
				$message .= "subscription to the ".$formData['from_name']." mailing list.\r\n\r\n";
			} else
			{
				$message .= $formData['Name'].' ('.$formData['Email'].') has been subscribed to'."\r\n";
				$message .= "the ".$formData['from_name']." mailing list.\r\n\r\n";
			}
		} elseif($action == 'unsubscribe')
		{
			$message .= $formData['Name'].' ('.$formData['Email'].') has been unsubscribed'."\r\n";
			$message .= "from the ".$formData['from_name']." mailing list.\r\n\r\n";
		}

		$entries = array();
		if(!isset($GLOBALS['secure_print']))
		{
			$entries = explode(",", $formData['secure_print_form_fields']);
			$GLOBALS['secure_print'] = $entries;
		} else
		{
			$entries = $GLOBALS['secure_print'];
		}

		foreach($entries as $form_field_name)
		{
			if(isset($formData[$form_field_name]))
			{
				$cleaned_name = $form_field_name;
				$cleaned_name = str_replace('_', ' ', $cleaned_name);
				$message .= $cleaned_name." : ".$formData[$form_field_name]."\r\n";
			}
		}

		global $zip_dump;
		if ($zip_dump &&
			(isset($formData['email_zip']) && preg_match("/(\.tar\.gz|\.hqx|\.zip|\.exe)/i", $formData['email_zip'])))
		{
			$message .= "\r\n";
			$message .= "Attachment: ".$zip_dump."/".$formData['email_zip']."\r\n";
			$message .= "\r\n";
		}

		# Print environmental variables. Useful for tracking and identification.

		$message .= "----\r\n";

		$message .= "Environmental (server and browser) variables :\r\n";
		$message .= "IP Number     : ".$_SERVER['REMOTE_ADDR']."\r\n";
		$message .= "Browser       : ".$_SERVER['HTTP_USER_AGENT']."\r\n";
		if(isset($_SERVER['HTTP_FORWARDED']))
		{
			$message .= "Proxy         : ".$_SERVER['HTTP_FORWARDED']."\r\n";
		}
		if(isset($_SERVER['HTTP_COOKIE']))
		{
			$message .= "Cookie        : ".$_SERVER['HTTP_COOKIE']."\r\n";
		}
		$message .= "Time:         : ".date("r")."\r\n";

		sendMail($formData['Recipient'], $subject, $message, $headers);
	}
}

###################

function their_email()
{
	global $formData, $path_to_data_files;

	if($formData['Email'] && $formData['from_email'])
	{

		$email_text_file = $formData['email_text_file'];
		if (!$email_text_file) {
			$email_text_file = "data/myemail.txt";
		}

		$handle = @fopen($path_to_data_files.'/'.$email_text_file, 'r');
		if($handle)
		{
			while(!feof($handle))
			{
				$line = fgets($handle, 1024);
				$email_text_file_data[] = trim($line);
			}
			fclose($handle);
		}

		$from = $formData['from_email'];
		$to = '"'.$formData['Name'].'" <'.$formData['Email'].'>';

		$headers = "";
		$headers .= "From: ".$formData['from_name'].' <'.$formData['from_email'].'>'."\r\n";

		$mailSubject = "";

		if($formData['Subject'])
		{
			$mailSubject = $formData['Subject'];
		} else
		{
			global $subject;
			$mailSubject = $subject;
		}

		$message = "";
		$message .= "Dear ".$formData['Name']."\r\n";
		$message .= "\r\n";

		if($formData['action'] == 'subscribe')
		{
			if(isset($GLOBALS['duplic']) && $GLOBALS['duplic'])
			{
				$message .= $formData['Name'].' ('.$formData['Email'].') has already been subscribed'."\r\n";
				$message .= " to the ".$formData['from_name']." mailing list.\r\n";
			} else
			{
				global $cgi_url;
				$message .= "Thank you for subscribing to our mailing list. If you want to\r\n";
				$message .= "unsubscribe at any time just click on this link:\r\n\r\n";
				$message .= $cgi_url."?Email=".$formData['Email']."&action=unsubscribe\r\n\r\n";
			}
		} elseif($formData['action'] == 'unsubscribe')
		{
				$message .= "You have been unsubscribed from the ".$formData['from_name']." mailing list. If\r\n";
				$message .= "you want to re-subscribe at any time you may do so at our website.\r\n";
				$message .= "Thank you.\r\n\r\n";

		}

		$message .= "\r\n";

		# Print their form data in email...
		$message .= "Here is the data you submitted. It may be incomplete for security reasons:\r\n";

		// $formData['Card_number'] = '';
		// $formData['Card_expiry_month'] = '';
		// $formData['Card_expiry_year'] = '';

		# print the raw form input, as specified earlier in the script:
		if (!$GLOBALS['secure_print'])
		{
        	$GLOBALS['secure_print'] = explode(",", $formData['secure_print_form_fields']);
		} # End if

		global $secure_print;

		foreach($secure_print as $value)
		{
			if($formData[$value])
			{
				# Make the names look more presentable (get rid of '_')
				$cleaned_name = $value;
				$cleaned_name = str_replace('_', ' ', $cleaned_name);

				$message .= $cleaned_name." : ".$formData[$value]."\r\n";
			}
		}

		$message .= "----\r\n";

		$message .= "\r\n";

		foreach($email_text_file_data as $line)
		{
			$message .= $line."\r\n";
		}

		$message .= "\r\n";
		$message .= "----\r\n";
		$message .= "\r\n";

		# base64encode and send a zip file with the mail, if specified in the form :
		global $zip_dump;
		if ($zip_dump &&
			(isset($formData['email_zip']) && preg_match("/(\.tar\.gz|\.hqx|\.zip|\.exe)/i", $formData['email_zip'])))
		{
			$binary = get_binary_file();
			if ($binary) {
				$binary_name = $formData['email_zip'];
				# look for path-info before filename
				$pos = strrpos($binary_name,"/");
				if ($pos !== FALSE) {
					# found path-info, cut it
					$binary_name = substr($binary_name,$pos);
				}
				$binary = chunk_split(base64_encode($binary));
				$boundary = md5(uniqid(time()));
				$headers .= "MIME-Version: 1.0"."\r\n";
				$headers .= "Content-Type: multipart/mixed; boundary=$boundary"."\r\n";
				$headers .= "Content-Transfer-Encoding: 7bit"."\r\n";
				$headers .= "\r\nThis is a multi-part message in MIME format"."\r\n";
				$headers .= "--$boundary"."\r\n";
				$headers .= "Content-Type: text/plain; charset=\"iso-8859-1\""."\r\n";
				$headers .= "Content-Transfer-Encoding: 7bit"."\r\n";
				$headers .= "\r\n[put_message_in_headers]"."\r\n";
				$headers .= "--$boundary"."\r\n";
				$headers .= "Content-Type: application/octet-stream; name=\"$binary_name\""."\r\n";
				$headers .= "Content-Transfer-Encoding: base64"."\r\n";
				$headers .= "Content-Disposition: attachment; name=\"$binary_name\""."\r\n";
				$headers .= "\r\n$binary"."\r\n";
				$headers .= "--$boundary--"."\r\n";
			}
		}

		sendMail($to, $mailSubject, $message, $headers);
	}
}


# Subscribe or unsubscribe the customer's email address
# to or from the email database :

function subscription()
{
	global $formData, $path_to_data_files, $DELIM2;

	$action = $formData['action'];
	$email = $formData['Email'];

	$subscriber_file = $formData['subscriber_file'];
	if (!$subscriber_file) {
		$subscriber_file = "data/subscriber.dat";
	}

	$subsFile = $path_to_data_files.'/'.$subscriber_file;

	if($action == 'subscribe')
	{
		if(file_exists($subsFile))
		{
			$content = file($subsFile);
			foreach($content as $line)
			{
				$smail = explode($DELIM2, $line);
				if(count($smail) > 1 && trim($smail[1]) == trim($email))
				{
					$GLOBALS['duplic'] = 1;
					break;
				}
			}

			if(!isset($GLOBALS['duplic']) || !$GLOBALS['duplic'])
			{
				$handle = @fopen($subsFile, "a");
				if($handle)
				{
					fwrite($handle, $formData['Name'].$DELIM2.$email."\n");
				}
				fclose($handle);
			}
		}
	} elseif($action == 'unsubscribe')
	{
		global $lockit;

		if(file_exists($subsFile))
		{
			$content = file($subsFile);
			$handle = @fopen($subsFile, "w");
			if($handle)
			{
				if($lockit)
				{
					flock($handle, LOCK_EX);
				}
				foreach($content as $line)
				{
					$smail = explode($DELIM2, $line);
					if(count($smail) != 2 || trim($smail[1]) != trim($email))
					{
						fwrite($handle, $line."\n");
					}
				}
				if($lockit)
				{
					flock($handle, LOCK_UN);
				}
				fclose($handle);
			}
		}
	}
}


######################

# Cookie get or set :

function cookie_bake()
{
	global $formData, $expires, $path, $domain, $secure;
	if(isset($formData['ID']))
	{
		setcookie("ID", $formData['ID'], time() - $expires * 3600 * 24, $path, $domain, $secure);
	}
}

# Sub routine to save data into pipe-delimited database format :

function save_delimited()
{
	global $DELIM2, $formData, $path_to_data_files, $lockit, $DELIM;

	$save_file = $formData['save_file'];
	if (!$save_file) {
		$save_file = "data/datafile.dat";
	}

	$dataToWrite = "";
	$referer = $_SERVER['HTTP_REFERER'];

	$dt = time();
	$cdate = date("l, F jS, Y", $dt);
	$ctime = date("H:i:s T(\G\M\T0)", $dt);

	$dataToWrite = $ctime;
	$dataToWrite .= $DELIM2.$cdate;
	$dataToWrite .= $DELIM2.$referer;
	$dataToWrite .= $DELIM2.$_SERVER['REMOTE_ADDR'];
	if(isset($_COOKIE['ID']))
	{
		$dataToWrite .= $DELIM2.$_COOKIE['ID'];
	}
	if(isset($_SERVER['HTTP_COOKIE']))
	{
		$dataToWrite .= $DELIM2.$_SERVER['HTTP_COOKIE'];
	}
	$dataToWrite .= $DELIM2;

	if($formData['insecure_save_form_fields'])
	{
		$insecure_save = explode(",", $formData['insecure_save_form_fields']);
		foreach($insecure_save as $name)
		{
			$dataToWrite .= $formData[$name].$DELIM2;
		}
	}

	$handle = @fopen($path_to_data_files.'/'.$save_file, 'a');
	if($handle)
	{
		if($lockit)
		{
			flock($handle, LOCK_EX);
		}
		fwrite($handle, $dataToWrite);
		fwrite($handle, "\n");
		if($lockit)
		{
			flock($handle, LOCK_UN);
		}
		fclose($handle);
	}
}

### Email a zip file

function get_binary_file()
{
	global $formData, $zip_dump;
	$binaryData = "";
	$handle = @fopen($zip_dump."/".$formData['email_zip'], "rb");
	if($handle)
	{
		while(!feof($handle))
		{
			$binaryData .= fread($handle, 4096);
		}
		fclose($handle);
	}
	return $binaryData;
}

### Check for email-injection symptoms

function check_email_injection($value)
{
	# look for CR / LF
	if (eregi("\r",$value) || eregi("\n",$value))
	{
		return TRUE;
	} else {
		return FALSE;
	}
}

######## SMS Message #############

function sms_message() 
{

	global $user, $password, $api_id, $baseurl, $to;

// Replace redundant or unimportant text
$extra_info = eregi_replace(" we | on | be | for | by | in | an | my | me | am | you | and | is | a | the | I | to | of "," ",$message);

// Prepare text message. Put form items in:
$text = "$Subject $Name $Email $Phone $Address $City $State $Zip : $extra_info";

// Remove excessive blank spaces
$text = ereg_replace("  "," ",$text);

// Find the number of characters, and reduce it to 160 if it's excessive, as we have a limit.
$number_of_characters = strlen($text);
if($number_of_characters > 160) {
$number_of_excessive_characters = $number_of_characters - 160;
// Remove one character sequentially from the end the text string, until we get 158 (remove two more for luck)
$counter = 0;
while($counter <= $number_of_excessive_characters + 2) {
 $text = substr($text, 0, -1);
	$counter++;
	}
    } // End if


// Encode the SMS
$text = urlencode("$text");

// auth call
$url = "$baseurl/http/auth?user=$user&password=$password&api_id=$api_id";
// do auth call
$ret = file($url);
// split our response. return string is on first line of the data returned
$sess = split(":",$ret[0]);
if ($sess[0] == "OK") {
$sess_id = trim($sess[1]); // remove any whitespace
$url = "$baseurl/http/sendmsg?session_id=$sess_id&to=$to&text=$text";
// do sendmsg call
$ret = file($url);
$send = split(":",$ret[0]);

// Put success or failure of SMS into a scalar
 if ($send[0] == "ID") {
 $SMS_result = "Text message send success. Message ID: " . $send[1];
	     }
 	 else {  $SMS_result = "Text message send failed!";
       		} 
if(!$SMS_result) { $SMS_result = "SMS authentication failure: ". $ret[0]; 
 	 }
   }

// Add SMS scalar to email content.
$message = $message . "\n\n" . $SMS_result . "\n\n";

}

################ Final URls to present to browser:

function _array_to_urlstring($data)
{
	$rv = "";
	$first = TRUE;
	foreach($data as $key => $value)
	{
		if($first)
		{
			$rv .= '&';
		}
		$rv .= $key.'='.urlencode($value);
	}
	return $rv;
}

function end_url()
{
	global $formData, $total;

	# Re-direct browser to FILE to download, if requested:
	if($formData['download_url'] && isset($formData['download']) && $formData['download'] !== "none")
	{
		if(!headers_sent())
		{
			header("Location: ".$formData['download_url'].'/'.$formData['download']);
		}
		return;
	} elseif($formData['second_script_url'])
	{
		if(!headers_sent())
		{
			header("Location: ".$formData['second_script_url'].'?'._array_to_urlstring($formData));
		}
		return;
	} elseif($formData['thanks_url'])
	{
		if(!headers_sent())
		{
			header("Location: ".$formData['thanks_url']);
		}
		return;
	} elseif($formData['Card_number'] || $total)
	{
		print_order_receipt();
	} else
	{
		if(!headers_sent())
		{
			header('Content-Type: text/html');
		}
		print "<h3>The form information has been sent.</h3>\n";
		exit();
	} # End else

} # End sub


?>